/*
   Purpose:      Demonstrates exception handling using try, catch and finally for both standard and custom exceptions.
   Note:         To get the standard exception (NumberFormatException is used), provide invalid value for the command line argument <deposit amount>, for example - abc
   Usage:        java CreateAccount <deposit amount>
   Note: 1000 was used for sample output
   Sample Output:

   accont balance = $1000.0
   primary savings account interest = 2.0%
   primary savings balance with interest applied = $1020.0

   withdrawing amount $1021.0.....
   Caught: NotEnoughMoneyException - You don't have sufficient funds to withdraw $1021.0
   Your current balance is $1020.0
*/
class SavingsAccount
{
   private double balance;
   double interest;

   SavingsAccount()
   {
      balance = 0.0;
   }

   SavingsAccount(double initial)
   {
      balance = initial;
   }

   public double getBalance()
   {
      return balance;
   }

   private double setBalance(double amount)
   {
      balance = amount;
      return balance;
   }

   public double calculateInterest()
   {
      interest *= balance/100;
      balance += interest;
      return balance;
   }

   public double calculateInterest(double bonus)
   {
      interest *= balance/100;
      balance += interest + bonus;
      return balance;
   }

   public void deposit(double amount)
   {
        balance += amount;
    }

   public void withdraw(double amount) throws NotEnoughMoneyException
   {

       double d;
       d = balance;
       d -= amount;

       if (d < 0)
       {
          throw new NotEnoughMoneyException();
       }
       else
       {
          setBalance(d);
       }
    }
}

class NotEnoughMoneyException extends Exception
{
     public NotEnoughMoneyException()
     {
         return;
    }
}

class CreateAccountException
{
   public static void main(String[] args)
   {
      double deposit;

      if (args.length > 0)
      {
          try
          {
              deposit = Double.parseDouble(args[0]);
          }
          catch (NumberFormatException e)
          {
              System.out.println("Caught: " + e + " You provided deposit as '" + args[0] + "', must be a number");
              return;
          }
         }
         else
         {
          System.out.println("You must provide a deposit amount");
          return;
       }

      SavingsAccount mySavings = new SavingsAccount(deposit);

      System.out.println("accont balance = $" + mySavings.getBalance());

      // setting the interest (%)
      mySavings.interest = 2;

      System.out.println("primary savings account interest = " + mySavings.interest + "%");

        System.out.println("primary savings balance with interest applied = $" + mySavings.calculateInterest());

        // DEMO: catching custom exception NotEnoughMoney when trying to overdraw the account.
        // for the demo purposes, the account is always overdrawn
        double balance = mySavings.getBalance();
        double amount = balance + 1;

        try
        {
         System.out.println("\nwithdrawing amount $" + (balance + 1) + ".....");
         mySavings.withdraw(amount);
        }
        catch (NotEnoughMoneyException e)
        {
         System.out.println("Caught: " + e + " - You don't have sufficient funds to withdraw amount $" + amount);
        }
        //printing account balance
        finally
        {
            System.out.println("Your current balance is $" + mySavings.getBalance());
        }
   }
}